@@ -74,6 +74,7 @@ int main(int argc, char *argv[])
 	int brief = 0;
 	int force = 0;
 	int test = 0;
+	int export = 0;
 	int assume_clean = 0;
 	char *symlinks = NULL;
 	/* autof indicates whether and how to create device node.
@@ -101,10 +102,12 @@ int main(int argc, char *argv[])
 	int re_add = 0;
 	char *shortopt = short_options;
 	int dosyslog = 0;
+	int rebuild_map = 0;
 	int auto_update_home = 0;
 
 	int copies;
 	int print_help = 0;
+	FILE *outf;
 
 	int mdfd = -1;
 
@@ -157,6 +160,9 @@ int main(int argc, char *argv[])
 			}
 			continue;
 
+		case 'Y': export++;
+			continue;
+
 		case HomeHost:
 			homehost = optarg;
 			continue;
@@ -174,7 +180,7 @@ int main(int argc, char *argv[])
 		switch(opt) {
 		case '@': /* just incase they say --manage */
 			newmode = MANAGE;
-			shortopt = short_bitmap_auto_options;
+			shortopt = short_bitmap_options;
 			break;
 		case 'a':
 		case 'r':
@@ -182,7 +188,7 @@ int main(int argc, char *argv[])
 		case ReAdd: /* re-add */
 			if (!mode) {
 				newmode = MANAGE;
-				shortopt = short_bitmap_auto_options;
+				shortopt = short_bitmap_options;
 			}
 			break;
 
@@ -190,7 +196,12 @@ int main(int argc, char *argv[])
 		case 'B': newmode = BUILD; shortopt = short_bitmap_auto_options; break;
 		case 'C': newmode = CREATE; shortopt = short_bitmap_auto_options; break;
 		case 'F': newmode = MONITOR;break;
-		case 'G': newmode = GROW; shortopt = short_bitmap_auto_options; break;
+		case 'G': newmode = GROW;
+			shortopt = short_bitmap_options;
+			break;
+		case 'I': newmode = INCREMENTAL; break;
+		case AutoDetect:
+			newmode = AUTODETECT; break;
 
 		case '#':
 		case 'D':
@@ -201,6 +212,7 @@ int main(int argc, char *argv[])
 		case 'S':
 		case 'o':
 		case 'w':
+		case 'W':
 		case 'K': if (!mode) newmode = MISC; break;
 		}
 		if (mode && newmode == mode) {
@@ -223,7 +235,7 @@ int main(int argc, char *argv[])
 			if (opt == 'c' && 
 			    ( strncmp(optarg, "--h", 3)==0 ||
 			      strncmp(optarg, "-h", 2)==0)) {
-				fputs(Help_config, stderr);
+				fputs(Help_config, stdout);
 				exit(0);
 			}
 
@@ -268,6 +280,8 @@ int main(int argc, char *argv[])
 		case 'C':
 		case 'F':
 		case 'G':
+		case 'I':
+		case AutoDetect:
 			continue;
 		}
 		if (opt == 1) {
@@ -320,6 +334,7 @@ int main(int argc, char *argv[])
 		case O(ASSEMBLE,AutoHomeHost):
 			auto_update_home = 1;
 			continue;
+		case O(INCREMENTAL, 'e'):
 		case O(CREATE,'e'):
 		case O(ASSEMBLE,'e'):
 		case O(MISC,'e'): /* set metadata (superblock) information */
@@ -515,6 +530,7 @@ int main(int argc, char *argv[])
 			continue;
 
 			/* now for the Assemble options */
+		case O(CREATE,'u'): /* uuid of array */
 		case O(ASSEMBLE,'u'): /* uuid of array */
 			if (ident.uuid_set) {
 				fprintf(stderr, Name ": uuid cannot be set twice.  "
@@ -582,6 +598,8 @@ int main(int argc, char *argv[])
 				continue;
 			if (strcmp(update, "homehost")==0)
 				continue;
+			if (strcmp(update, "devicesize")==0)
+				continue;
 			if (strcmp(update, "byteorder")==0) {
 				if (ss) {
 					fprintf(stderr, Name ": must not set metadata type with --update=byteorder.\n");
@@ -596,17 +614,24 @@ int main(int argc, char *argv[])
 
 				continue;
 			}
-			if (strcmp(update,"?") == 0 || strcmp(update, "help") == 0)
-				fprintf(stderr, Name ": ");
-			else
-				fprintf(stderr, Name ": '--update=%s' is invalid.  ", update);
-			fprintf(stderr, "Valid --update options are:\n"
-				"     'sparc2.2', 'super-minor', 'uuid', 'name', 'resync',\n"
-				"     'summaries', 'homehost', 'byteorder'.\n");
-			exit(2);
+			if (strcmp(update,"?") == 0 ||
+			    strcmp(update, "help") == 0) {
+				outf = stdout;
+				fprintf(outf, Name ": ");
+			} else {
+				outf = stderr;
+				fprintf(outf,
+					Name ": '--update=%s' is invalid.  ",
+					update);
+			}
+			fprintf(outf, "Valid --update options are:\n"
+		"     'sparc2.2', 'super-minor', 'uuid', 'name', 'resync',\n"
+		"     'summaries', 'homehost', 'byteorder', 'devicesize'.\n");
+			exit(outf == stdout ? 0 : 2);
 
 		case O(ASSEMBLE,NoDegraded): /* --no-degraded */
-			runstop = -1; /* --stop isn't allowed for --assemble, so we overload slightly */
+			runstop = -1; /* --stop isn't allowed for --assemble,
+				       * so we overload slightly */
 			continue;
 
 		case O(ASSEMBLE,'c'): /* config file */
@@ -624,6 +649,7 @@ int main(int argc, char *argv[])
 		case O(ASSEMBLE,'s'): /* scan */
 		case O(MISC,'s'):
 		case O(MONITOR,'s'):
+		case O(INCREMENTAL,'s'):
 			scan = 1;
 			continue;
 
@@ -698,6 +724,7 @@ int main(int argc, char *argv[])
 		case O(MANAGE,'f'): /* set faulty */
 			devmode = 'f';
 			continue;
+		case O(INCREMENTAL,'R'):
 		case O(MANAGE,'R'):
 		case O(ASSEMBLE,'R'):
 		case O(BUILD,'R'):
@@ -740,6 +767,7 @@ int main(int argc, char *argv[])
 		case O(MISC,'X'):
 		case O(MISC,'o'):
 		case O(MISC,'w'):
+		case O(MISC,'W'):
 			if (devmode && devmode != opt &&
 			    (devmode == 'E' || (opt == 'E' && devmode != 'Q'))) {
 				fprintf(stderr, Name ": --examine/-E cannot be given with -%c\n",
@@ -789,9 +817,15 @@ int main(int argc, char *argv[])
 			backup_file = optarg;
 			continue;
 
-		case O(GROW,'b'):
 		case O(BUILD,'b'):
 		case O(CREATE,'b'): /* here we create the bitmap */
+			if (strcmp(optarg, "none") == 0) {
+				fprintf(stderr, Name ": '--bitmap none' only"
+					" support for --grow\n");
+				exit(2);
+			}
+			/* FALL THROUGH */
+		case O(GROW,'b'):
 			if (strcmp(optarg, "internal")== 0 ||
 			    strcmp(optarg, "none")== 0 ||
 			    strchr(optarg, '/') != NULL) {
@@ -816,6 +850,7 @@ int main(int argc, char *argv[])
 			bitmap_chunk = bitmap_chunk ? bitmap_chunk * 1024 : 512;
 			continue;
 
+		case O(GROW, WriteBehind):
 		case O(BUILD, WriteBehind):
 		case O(CREATE, WriteBehind): /* write-behind mode */
 			write_behind = DEFAULT_MAX_WRITE_BEHIND;
@@ -828,6 +863,10 @@ int main(int argc, char *argv[])
 				}
 			}
 			continue;
+
+		case O(INCREMENTAL, 'r'):
+			rebuild_map = 1;
+			continue;
 		}
 		/* We have now processed all the valid options. Anything else is
 		 * an error
@@ -856,8 +895,9 @@ int main(int argc, char *argv[])
 			case MISC     : help_text = Help_misc; break;
 			case MONITOR  : help_text = Help_monitor; break;
 			case GROW     : help_text = Help_grow; break;
+			case INCREMENTAL:help_text= Help_incr; break;
 			}
-		fputs(help_text,stderr);
+		fputs(help_text,stdout);
 		exit(0);
 	}
 
@@ -1110,6 +1150,11 @@ int main(int argc, char *argv[])
 			rv = 1;
 			break;
 		}
+		if (raiddisks == 0) {
+			fprintf(stderr, Name ": no raid-disks specified.\n");
+			rv = 1;
+			break;
+		}
 
 		if (bitmap_file) {
 			if (strcmp(bitmap_file, "internal")==0) {
@@ -1129,14 +1174,19 @@ int main(int argc, char *argv[])
 			rv = 1;
 			break;
 		}
+		if (raiddisks == 0) {
+			fprintf(stderr, Name ": no raid-disks specified.\n");
+			rv = 1;
+			break;
+		}
 
 		rv = Create(ss, devlist->devname, mdfd, chunk, level, layout, size<0 ? 0 : size,
 			    raiddisks, sparedisks, ident.name, homehost,
+			    ident.uuid_set ? ident.uuid : NULL,
 			    devs_found-1, devlist->next, runstop, verbose-quiet, force, assume_clean,
 			    bitmap_file, bitmap_chunk, write_behind, delay);
 		break;
 	case MISC:
-
 		if (devmode == 'E') {
 			if (devlist == NULL && !scan) {
 				fprintf(stderr, Name ": No devices to examine\n");
@@ -1165,7 +1215,8 @@ int main(int argc, char *argv[])
 								e->dev);
 							continue;
 						}
-						rv |= Detail(name, verbose>1?0:verbose+1, test, homehost);
+						rv |= Detail(name, verbose>1?0:verbose+1,
+							     export, test, homehost);
 						put_md_name(name);
 					}
 				} else	if (devmode == 'S' && scan) {
@@ -1210,13 +1261,18 @@ int main(int argc, char *argv[])
 			for (dv=devlist ; dv; dv=dv->next) {
 				switch(dv->disposition) {
 				case 'D':
-					rv |= Detail(dv->devname, brief?1+verbose:0, test, homehost); continue;
+					rv |= Detail(dv->devname,
+						     brief?1+verbose:0,
+						     export, test, homehost);
+					continue;
 				case 'K': /* Zero superblock */
 					rv |= Kill(dv->devname, force, quiet); continue;
 				case 'Q':
 					rv |= Query(dv->devname); continue;
 				case 'X':
 					rv |= ExamineBitmap(dv->devname, brief, ss); continue;
+				case 'W':
+					rv |= Wait(dv->devname); continue;
 				}
 				mdfd = open_mddev(dv->devname, 1);
 				if (mdfd>=0) {
@@ -1282,6 +1338,38 @@ int main(int argc, char *argv[])
 		} else
 			fprintf(stderr, Name ": no changes to --grow\n");
 		break;
+	case INCREMENTAL:
+		if (rebuild_map) {
+			RebuildMap();
+		}
+		if (scan) {
+			if (runstop <= 0) {
+				fprintf(stderr, Name
+			 ": --incremental --scan meaningless without --run.\n");
+				break;
+			}
+			rv = IncrementalScan(verbose);
+		}
+		if (!devlist) {
+			if (!rebuild_map && !scan) {
+				fprintf(stderr, Name
+					": --incremental requires a device.\n");
+				rv = 1;
+			}
+			break;
+		}
+		if (devlist->next) {
+			fprintf(stderr, Name
+			       ": --incremental can only handle one device.\n");
+			rv = 1;
+			break;
+		}
+		rv = Incremental(devlist->devname, verbose-quiet, runstop,
+				 ss, homehost, autof);
+		break;
+	case AUTODETECT:
+		autodetect();
+		break;
 	}
 	exit(rv);
 }
